local AddName, AddonTable = ...
local L = AddonTable.L

------------------------------
--		Options Equates		--
------------------------------
local Both, Enhanced, Base, Damage, DamageTaken, Melee, Ranged, Max, Average, Low = 3, 1, 2, 1, 2, 1, 2, 1, 2, 3

----------------------------------
--		Text Return Formats		--
----------------------------------
local Double_Percent_Format = { "%.2f%%", "%.2f%%", "%.2f/%.2f%%", }

--------------------------
--		Physical		--
--------------------------
-- Strength
function AddonTable.FunctionList.Strength(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Enhanced_Base
	local _, stat, posBuff, negBuff = UnitStat("Player", 1)
	local debuffColor = ""
	local returnText
	
	stat = stat + (stat * AddonTable.mysticTouch) + (stat * AddonTable.colossusSmash) + (stat * AddonTable.razorIce) + (stat * AddonTable.felSunder) + (stat * AddonTable.faeExposure)
	
	if negBuff < 0 then debuffColor = "|cffC41E3A" end

	if AddonTable.Band(EB, Enhanced) then returnText = debuffColor .. stat end
	if AddonTable.Band(EB, Base) then returnText = base end
	if AddonTable.Band(EB, Both) then returnText = debuffColor .. stat .. "|r/" .. base end
	if returnText == nil then returnText = 0 end			

	HUD:UpdateText(data, returnText)
end

-- Agility
function AddonTable.FunctionList.Agility(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Enhanced_Base
	local _, stat, posBuff, negBuff = UnitStat("Player", 2)
	local debuffColor = ""
	local returnText
	local base = stat
	
	stat = stat + (stat * AddonTable.mysticTouch) + (stat * AddonTable.colossusSmash) + (stat * AddonTable.razorIce) + (stat * AddonTable.felSunder) + (stat * AddonTable.faeExposure) + (stat * AddonTable.serenity)
	
	if negBuff < 0 then debuffColor = "|cffC41E3A" end

	if AddonTable.Band(EB, Enhanced) then returnText = debuffColor .. stat end
	if AddonTable.Band(EB, Base) then returnText = base end
	if AddonTable.Band(EB, Both) then returnText = debuffColor .. stat .. "|r/" .. base end	
	if returnText == nil then returnText = 0 end		

	HUD:UpdateText(data, returnText)
end

-- Attack Power
function AddonTable.FunctionList.AP(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Melee_Ranged
	local base, posBuff, negBuff = UnitAttackPower("player");
	local effectiveAP = base + posBuff + negBuff;
	local base, posBuff, negBuff = UnitRangedAttackPower("player");
	local effectiveRanged = base + posBuff + negBuff;	
	local debuffColor = ""
	local returnText
	
	if negBuff < 0 then debuffColor = "|cffC41E3A" end

	if AddonTable.Band(EB, Melee) then returnText = debuffColor .. effectiveAP end
	if AddonTable.Band(EB, Ranged) then returnText = debuffColor .. effectiveRanged end
	if AddonTable.Band(EB, Both) then returnText = debuffColor .. effectiveAP .. "|r/" .. debuffColor .. effectiveRanged end
	if returnText == nil then returnText = 0 end
	
	HUD:UpdateText(data, returnText)
end

-- Damage
function AddonTable.FunctionList.DMG(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Max_Average_Damage
	local lowDmg, hiDmg, offlowDmg, offhiDmg, posBuff, negBuff, percentmod = UnitDamage("player")
	local totalDamage = hiDmg
	local meleeDamageOH = offhiDmg
	local avgDamage = (hiDmg + lowDmg) / 2
	local lowDmg = lowDmg
	local speed = UnitRangedDamage("player")
	local debuffColor = ""
	local ohID = GetInventoryItemID("player", 17)
	local itemSubtype, ohsubId = 0, 0
	local returnText, decimals
	
	if options.Decimals_Damage == 0 then decimals = "%.0f"
	elseif options.Decimals_Damage == 1 then decimals = "%.1f"
	elseif options.Decimals_Damage == 2 then decimals = "%.2f"
	else decimals = "%.3f" end	
	
	local avgOhDmg = "/" .. (decimals):format((offhiDmg + offlowDmg) / 2)
	local lowOH = "/" .. (decimals):format(offlowDmg)	
	
	if speed > 0 then 
		local _, rlowDmg, rhiDmg, _, negBuff = UnitRangedDamage("player")
		totalDamage = rhiDmg
		avgDamage = (rhiDmg + rlowDmg) / 2
		avgOhDmg = ""
		lowDmg = rlowDmg
		lowOH = ""
	end
	
	if ohID then _,_,_,_,_,_, itemSubtype, _, _, _, _, _, ohsubId = GetItemInfo(ohID) end
	if ohID or ohsubId == 2 then 
		meleeDamageOH = meleeDamageOH + (meleeDamageOH * AddonTable.mysticTouch) + (meleeDamageOH * AddonTable.colossusSmash) + (meleeDamageOH * AddonTable.razorIce) + (meleeDamageOH * AddonTable.felSunder) + (meleeDamageOH * AddonTable.faeExposure) + (meleeDamageOH * AddonTable.serenity)
		meleeDamageOH = "/" .. (decimals):format(meleeDamageOH)
	else 
		meleeDamageOH = ""
		avgOhDmg = ""
		lowOH = ""
	end
	
	if options.Display_MainHand then meleeDamageOH = ""; avgOhDmg = ""; lowOH = "" end
	
	totalDamage = totalDamage + (totalDamage * AddonTable.mysticTouch) + (totalDamage * AddonTable.colossusSmash) + (totalDamage * AddonTable.razorIce) + (totalDamage * AddonTable.felSunder) + (totalDamage * AddonTable.faeExposure) + (totalDamage * AddonTable.serenity)
	
	if negBuff < 0 then debuffColor = "|cffC41E3A" end
	
	if AddonTable.Band(EB, Max) then returnText = debuffColor .. (decimals):format(totalDamage) .. meleeDamageOH end
	if AddonTable.Band(EB, Average) then returnText = debuffColor .. (decimals):format(avgDamage) .. avgOhDmg end
	if AddonTable.Band(EB, Low) then returnText = debuffColor .. (decimals):format(lowDmg) .. lowOH end
	if returnText == nil then returnText = 0 end	

	HUD:UpdateText(data, returnText)		
end

-- Weapon Speed
function AddonTable.FunctionList.weaponSpeed(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Melee_Ranged
	local mainSpeed, offSpeed = UnitAttackSpeed("player")
	local rangedSpeed = UnitRangedDamage("player")
	local returnText
	
	if offSpeed == nil then offSpeed = ""
	else offSpeed = "/" .. ("%.1f"):format(offSpeed) end
	
	if AddonTable.Band(EB, Melee) then returnText = ("%.2f"):format(mainSpeed) .. offSpeed end
	if AddonTable.Band(EB, Ranged) then returnText = ("%.1f"):format(rangedSpeed) end
	if AddonTable.Band(EB, Both) then returnText = ("%.2f"):format(mainSpeed) .. offSpeed .. "|r (" .. ("%.1f"):format(rangedSpeed) .. ")" end
	if returnText == nil then returnText = 0 end	
	
	HUD:UpdateText(data, returnText)
end

-- Energy Regen
function AddonTable.FunctionList.EnergyRegen(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Enhanced_Base
	local haste = GetHaste()
	local energyRegen = 0
	local returnText, decimals
	energyRegen = 10 + (haste * 0.1)
	
	if options.Decimals_Energy == 0 then decimals = "%.0f"
	elseif options.Decimals_Energy == 1 then decimals = "%.1f"
	elseif options.Decimals_Energy == 2 then decimals = "%.2f"
	else decimals = "%.3f" end	
	
	if AddonTable.Band(EB, Enhanced) then returnText = (decimals):format(energyRegen) end
	if AddonTable.Band(EB, Base) then returnText = (decimals):format(energyRegen) end
	if AddonTable.Band(EB, Both) then returnText = (decimals):format(energyRegen) .. " (" .. (decimals):format(energyRegen) .. ")" end
	if returnText == nil then returnText = 0 end		

	HUD:UpdateText(data, returnText)	
end

-- Stamina
function AddonTable.FunctionList.Stamina(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Enhanced_Base
	local _, stat, posBuff, negBuff = UnitStat("Player", 3)
	local debuffColor = ""
	local returnText
	
	if negBuff < 0 then debuffColor = "|cffC41E3A" end

	if AddonTable.Band(EB, Enhanced) then returnText = debuffColor .. stat end
	if AddonTable.Band(EB, Base) then returnText = stat end
	if AddonTable.Band(EB, Both) then returnText = debuffColor .. stat .. "|r (" .. stat .. ")" end
	if returnText == nil then returnText = 0 end		

	HUD:UpdateText(data, returnText)
end

-- Armor
function AddonTable.FunctionList.Armor(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Enhanced_Base
	local base, effectiveArmor, armor, posBuff, negBuff = UnitArmor("player")
	local debuffColor = ""
	
	if negBuff == nil then negBuff = 0 end
	if negBuff < 0 then debuffColor = "|cffC41E3A" end

	if AddonTable.Band(EB, Enhanced) then returnText = debuffColor .. effectiveArmor end
	if AddonTable.Band(EB, Base) then returnText = effectiveArmor end
	if AddonTable.Band(EB, Both) then returnText = debuffColor .. effectiveArmor .. "|r (" .. effectiveArmor .. ")" end
	if returnText == nil then returnText = 0 end		

	HUD:UpdateText(data, returnText)
end

-- Dodge
function AddonTable.FunctionList.Dodge(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Enhanced_Base
	local dodgeChance = GetDodgeChance("player")
	local returnText, decimals
	
	if options.Decimals_Dodge == 0 then decimals = "%.0f%%"
	elseif options.Decimals_Dodge == 1 then decimals = "%.1f%%"
	elseif options.Decimals_Dodge == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end

	if AddonTable.Band(EB, Enhanced) then returnText = (decimals):format(dodgeChance) end
	if AddonTable.Band(EB, Base) then returnText = (decimals):format(dodgeChance) end
	if AddonTable.Band(EB, Both) then returnText = (decimals):format(dodgeChance) .. " (" .. (decimals):format(dodgeChance) .. ")" end
	if returnText == nil then returnText = 0 end		

	HUD:UpdateText(data, returnText)
end

-- Parry
function AddonTable.FunctionList.Parry(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Enhanced_Base
	local parryChance = GetParryChance("player")
	local returnText, decimals
	
	if options.Decimals_Parry == 0 then decimals = "%.0f%%"
	elseif options.Decimals_Parry == 1 then decimals = "%.1f%%"
	elseif options.Decimals_Parry == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end	

	if AddonTable.Band(EB, Enhanced) then returnText = (decimals):format(parryChance) end
	if AddonTable.Band(EB, Base) then returnText = (decimals):format(parryChance) end
	if AddonTable.Band(EB, Both) then returnText = (decimals):format(parryChance) .. " (" .. (decimals):format(parryChance) .. ")" end
	if returnText == nil then returnText = 0 end	

	HUD:UpdateText(data, returnText)
end

-- Block
function AddonTable.FunctionList.Block(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Enhanced_Base
	local blockChance = GetBlockChance("player")
	local returnText, decimals
	
	if options.Decimals_Block == 0 then decimals = "%.0f%%"
	elseif options.Decimals_Block == 1 then decimals = "%.1f%%"
	elseif options.Decimals_Block == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end		

	if AddonTable.Band(EB, Enhanced) then returnText = (decimals):format(blockChance) end
	if AddonTable.Band(EB, Base) then returnText = (decimals):format(blockChance) end
	if AddonTable.Band(EB, Both) then returnText = (decimals):format(blockChance) .. " (" .. (decimals):format(blockChance) .. ")" end
	if returnText == nil then returnText = 0 end		

	HUD:UpdateText(data, returnText)
end

-- Absorb
function AddonTable.FunctionList.Absorb(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Enhanced_Base
	local absorb = UnitGetTotalAbsorbs("player")

	if AddonTable.Band(EB, Enhanced) then returnText = absorb end
	if AddonTable.Band(EB, Base) then returnText = absorb end
	if AddonTable.Band(EB, Both) then returnText = absorb .. " (" .. absorb .. ")" end
	if returnText == nil then returnText = 0 end	

	HUD:UpdateText(data, returnText)
end

-- Stagger
function AddonTable.FunctionList.Stagger(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Enhanced_Base
	local stagger = C_PaperDollInfo.GetStaggerPercentage("player")
	local returnText, decimals
	
	if options.Decimals_Stagger == 0 then decimals = "%.0f%%"
	elseif options.Decimals_Stagger == 1 then decimals = "%.1f%%"
	elseif options.Decimals_Stagger == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end		

	if AddonTable.Band(EB, Enhanced) then returnText = (decimals):format(stagger) end
	if AddonTable.Band(EB, Base) then returnText = (decimals):format(stagger) end
	if AddonTable.Band(EB, Both) then returnText = (decimals):format(stagger) .. " (" .. (decimals):format(stagger) .. ")" end
	if returnText == nil then returnText = 0 end		

	HUD:UpdateText(data, returnText)
end
